/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright 2014 Yamato Suzuki
 *
 */

#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/csma-module.h"
#include "ns3/applications-module.h"

#include "ns3/BitTorrentTracker.h"
#include "ns3/BitTorrentClient.h"
#include "ns3/MetainfoFile.h"

using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("P2P_SimulationExample");

/*
static void
CwndChange (uint32_t oldCwnd, uint32_t newCwnd)
{
  NS_LOG_UNCOND (Simulator::Now ().GetSeconds () << "\t" << newCwnd);
}

static void
RxDrop (Ptr<const Packet> p)
{
  NS_LOG_UNCOND ("RxDrop at " << Simulator::Now ().GetSeconds ());
}
*/

int 
main (int argc, char *argv[])
{
  LogComponentEnable ("P2P_SimulationExample", LOG_LEVEL_INFO);
  //  LogComponentEnable ("BitTorrentTracker", LOG_LEVEL_INFO);
  LogComponentEnable ("BitTorrentClient", LOG_LEVEL_INFO);
  LogComponentEnable ("BitTorrentConnectionManager", LOG_LEVEL_INFO);

  LogComponentEnable ("ClientDatabase", LOG_LEVEL_FUNCTION);

  LogComponentEnable ("ConnectionStrategyBase", LOG_LEVEL_INFO);
  LogComponentEnable ("FullRandomConnectionStrategy", LOG_LEVEL_INFO);

  LogComponentEnable ("ListeningPeerConnection", LOG_LEVEL_INFO);

  LogComponentEnable ("PeerConnection", LOG_LEVEL_FUNCTION);

  LogComponentEnable ("FullRandomRequestStrategy", LOG_LEVEL_FUNCTION);

  NS_LOG_INFO("Creating Topology");

  NodeContainer trackerBackboneNodes;
  trackerBackboneNodes.Create (2);

  PointToPointHelper pointToPoint;
  pointToPoint.SetDeviceAttribute ("DataRate", StringValue ("5Mbps"));
  pointToPoint.SetChannelAttribute ("Delay", StringValue ("2ms"));

  NetDeviceContainer trackerBackboneDevices;
  trackerBackboneDevices = pointToPoint.Install (trackerBackboneNodes);


  NodeContainer peerCsmaNodes1;
  NodeContainer peerCsmaNodes2;
  peerCsmaNodes1.Add (trackerBackboneNodes.Get(1));
  peerCsmaNodes2.Add (trackerBackboneNodes.Get(1));

  peerCsmaNodes1.Create(1);
  peerCsmaNodes2.Create(1);

  CsmaHelper csma;
  csma.SetChannelAttribute ("DataRate", StringValue("100Mbps"));
  csma.SetChannelAttribute ("Delay", TimeValue(NanoSeconds(6560)));

  NetDeviceContainer peerCsmaDevices1;
  NetDeviceContainer peerCsmaDevices2;
  peerCsmaDevices1 = csma.Install (peerCsmaNodes1);
  peerCsmaDevices2 = csma.Install (peerCsmaNodes2);


  InternetStackHelper stack;
  stack.Install (trackerBackboneNodes);
  stack.Install (peerCsmaNodes1.Get(1));
  stack.Install (peerCsmaNodes2.Get(1));

  Ipv4AddressHelper address;
  address.SetBase ("10.1.1.0", "255.255.255.252");
  Ipv4InterfaceContainer trackerBackboneInterfaces = address.Assign (trackerBackboneDevices);
  
  address.SetBase ("172.16.1.0", "255.255.255.252");
  Ipv4InterfaceContainer peerCsmaInterfaces1 = address.Assign (peerCsmaDevices1);
  
  address.SetBase ("172.16.2.0", "255.255.255.252");
  Ipv4InterfaceContainer peerCsmaInterfaces2 = address.Assign (peerCsmaDevices2);


  // Tracker Server Configure
  Address trackerAddress = InetSocketAddress(trackerBackboneInterfaces.GetAddress(0), 6969);

  Ptr<BitTorrentTracker> tracker = Create<BitTorrentTracker>();
  trackerBackboneNodes.Get(0)->AddApplication(tracker);
  tracker->SetStartTime(Seconds(10.));
  tracker->SetStopTime(Seconds(280.));

  // Generate Metainfo File
  Ptr<MetainfoFile> info = Create<MetainfoFile>(trackerAddress, 10 * 1024 * 1024, 16 * 1024);

  // Client Application Configure
  Ptr<BitTorrentClient> client1 = Create<BitTorrentClient>(info, true);
  peerCsmaNodes1.Get(1)->AddApplication(client1);
  client1->SetStartTime(Seconds(20.));
  client1->SetStopTime(Seconds(30.));

  Ptr<BitTorrentClient> client2 = Create<BitTorrentClient>(info);
  peerCsmaNodes2.Get(1)->AddApplication(client2);
  client2->SetStartTime(Seconds(10.));
  client2->SetStopTime(Seconds(40.));

  /*
  uint16_t sinkPort = 8080;
  Address sinkAddress (InetSocketAddress (interfaces.GetAddress (1), sinkPort));
  PacketSinkHelper packetSinkHelper ("ns3::TcpSocketFactory", InetSocketAddress (Ipv4Address::GetAny (), sinkPort));
  ApplicationContainer sinkApps = packetSinkHelper.Install (nodes.Get (1));
  sinkApps.Start (Seconds (0.));
  sinkApps.Stop (Seconds (20.));

  Ptr<Socket> ns3TcpSocket = Socket::CreateSocket (nodes.Get (0), TcpSocketFactory::GetTypeId ());
  ns3TcpSocket->TraceConnectWithoutContext ("CongestionWindow", MakeCallback (&CwndChange));

  Ptr<MyApp> app = CreateObject<MyApp> ();
  app->Setup (ns3TcpSocket, sinkAddress, 1040, 1000, DataRate ("1Mbps"));
  nodes.Get (0)->AddApplication (app);
  app->SetStartTime (Seconds (1.));
  app->SetStopTime (Seconds (20.));

  //  devices.Get (1)->TraceConnectWithoutContext ("PhyRxDrop", MakeCallback (&RxDrop));

  */

  Ipv4GlobalRoutingHelper::PopulateRoutingTables();

  pointToPoint.EnablePcapAll("p2p-tracker");
  csma.EnablePcapAll("p2p-local");

  Simulator::Stop (Seconds (300));
  Simulator::Run ();
  Simulator::Destroy ();

  return 0;
}

