/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include "ns3/log.h"
#include "ns3/simulator.h"
#include "ns3/tcp-socket-factory.h"
#include "ns3/ipv4.h"

#include "BitTorrentClient.h"

namespace ns3 {

NS_LOG_COMPONENT_DEFINE("BitTorrentClient");
NS_OBJECT_ENSURE_REGISTERED(BitTorrentClient);

BitTorrentClient::BitTorrentClient(Ptr<MetainfoFile> metainfo, bool isSeeder)
{
  NS_LOG_FUNCTION(this);

  m_metainfoFile      = metainfo;
  m_clientDB          = Create<ClientDatabase> (m_metainfoFile, isSeeder);
  m_trackerManager    = Create<TrackerManager> (m_metainfoFile->trackerAddress(), this, m_clientDB);
  m_connectionManager = Create<ConnectionManager> (this, m_clientDB);
}

BitTorrentClient::~BitTorrentClient()
{
  NS_LOG_FUNCTION(this);
}

// ====================================================================================================

void
BitTorrentClient::setAsn(uint16_t asn)
{
  m_clientDB->setAsn(asn);
}

// ====================================================================================================

Ptr<Socket>
BitTorrentClient::CreateTcpSocket(void)
{
  return Socket::CreateSocket(GetNode(), TcpSocketFactory::GetTypeId());
}

Ipv4Address
BitTorrentClient::ClientIpv4Address(void)
{
  const uint32_t interfaceNumber = 1;
  const uint32_t addressIndex    = 0;

  return GetNode()->GetObject<Ipv4>()->GetAddress(interfaceNumber, addressIndex).GetLocal();
}

// ====================================================================================================

void
BitTorrentClient::DoDispose(void)
{
  NS_LOG_FUNCTION(this);

  Application::DoDispose();
}


void
BitTorrentClient::StartApplication(void)
{
  NS_LOG_FUNCTION(this);

  std::stringstream ss;
  ss << "NODE_ID:" << GetNode()->GetId();
  m_clientDB->setPeerId(ss.str());

  CLIENT_LOG_INFO("Start Application");

  m_trackerManager->StartRequest();
  m_connectionManager->StartConnectionManager();

}

void
BitTorrentClient::StopApplication(void)
{
  NS_LOG_FUNCTION(this);

  CLIENT_LOG_INFO("Stop Application");

  m_trackerManager->StopRequest();
  m_connectionManager->StopConnectionManager();
}

void
BitTorrentClient::CLIENT_LOG_INFO(std::string info)
{
  NS_LOG_INFO("[" << m_clientDB->getPeerId() << "] " <<
              Simulator::Now().GetSeconds() << "[s] " << info);
}

}

