/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef BITTORRENT_PROTOCOL_H
#define BITTORRENT_PROTOCOL_H

#include "ns3/object.h"

namespace ns3 {

class Socket;

class BitTorrentProtocol : public Object
{
public:
  enum MessageType {CHOKE          = 0x00,
                    UNCHOKE        = 0x01,
                    INTERESTED     = 0x02,
                    NOT_INTERESTED = 0x03,
                    HAVE           = 0x04,
                    BITFIELD       = 0x05,
                    REQUEST        = 0x06,
                    PIECE          = 0x07,
                    CANCEL         = 0x08,
                    KEEP_ALIVE     = 0xf0,
                    INCOMPLETE     = 0xf1,};

  static BitTorrentProtocol& GetInstance(void);
protected:
  BitTorrentProtocol();

public:
  bool SendHandshakeMessage(Ptr<Socket> socket, std::string peerId, std::string infoHash);

  bool SendKeepAliveMessage(Ptr<Socket> socket);
  bool SendChokeMessage(Ptr<Socket> socket);
  bool SendUnchokeMessage(Ptr<Socket> socket);
  bool SendInterestedMessage(Ptr<Socket> socket);
  bool SendNotInterestedMessage(Ptr<Socket> socket);
  bool SendHaveMessage(Ptr<Socket> socket, uint32_t pieceIndex);
  bool SendBitfieldMessage(Ptr<Socket> socket, std::string bitfield);
  bool SendRequestMessage(Ptr<Socket> socket, uint32_t pieceIndex, uint32_t begin, uint32_t requestLength);
  bool SendPieceMessage(Ptr<Socket> socket, uint32_t pieceIndex, uint32_t begin, std::string blockData);
  bool SendCancelMessage(Ptr<Socket> socket, uint32_t pieceIndex, uint32_t begin, uint32_t requestLength);

public:
  bool ParseHandshakeMessage(std::string message, std::string* peerId, std::string* infoHash);

  bool ParseHaveMessage(std::string message, uint32_t* pieceIndex);
  bool ParseBitfieldMessage(std::string message, std::string* bitfield);
  bool ParseRequestMessage(std::string message, uint32_t* pieceIndex, uint32_t* begin, uint32_t* requestLength);
  bool ParsePieceMessage(std::string message, uint32_t* pieceIndex, uint32_t* begin, std::string* blockData);
  bool ParseCancelMessage(std::string message, uint32_t* pieceIndex, uint32_t* begin, uint32_t* requestLength);

public:
  BitTorrentProtocol::MessageType CheckMessageType(std::string message);
  uint32_t getIdealMessageSize(std::string message);

private:
  std::string getByteString(uint32_t value);
  std::string getByteString(uint8_t value);

  uint32_t getUint32_t(std::string data);

private:
  std::string m_protocolId;

};

}

#endif /* BITTORRENT_PROTOCOL_H */

