/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef CLIENT_DATABASE_H
#define CLIENT_DATABASE_H

#include "ns3/object.h"

namespace ns3 {

class MetainfoFile;
class PeerInformation;
class ConnectionManager;

class ClientDatabase : public Object
{
public:
  enum Status {STARTED, STOPPED, COMPLETED};

  ClientDatabase(Ptr<MetainfoFile> metainfo, bool isSeeder = false);
private:
  ClientDatabase();

public:
  void setConnectionManager(Ptr<ConnectionManager> connectionManager);

  ClientDatabase::Status getStatus(void) const;

  std::string getPeerId(void) const;
  void setPeerId(std::string peerId);

  std::string getInfoHash(void) const;
  uint32_t getPieceLength(void) const;

  void     setListeningPort(uint16_t port);
  uint16_t listeningPort(void);

  uint32_t uploaded(void);
  uint32_t downloaded(void);
  uint32_t left(void);

  const std::vector<bool>& getBitfield(void) const;
  std::string getBitfieldString(void) const;
  bool havePiece(uint32_t pieceIndex) const;
  bool receiveNewPiece(uint32_t pieceIndex, uint32_t begin, const std::string& blockData);

public:
  void setAsn(uint16_t asn);
  uint16_t getAsn(void) const;

public:
  const std::vector<PeerInformation>& getPeerList(void) const;
  void AddToPeerList(std::string peer_id, std::string ip, uint16_t port, uint16_t asn);
  void ClearPeerList(void);

private:
  void UpdateStatus(void);

private:
  Status      m_status;
  std::string m_peerId;
  std::string m_infoHash;

  uint32_t    m_fileLength;
  uint32_t    m_pieceLength;
  uint32_t    m_pieceIndexLength;

  std::vector<bool> m_bitfield;

  uint16_t    m_listeningPort;

  uint32_t    m_uploaded;
  uint32_t    m_downloaded;
  uint32_t    m_left;

  std::vector<PeerInformation> m_peerList;

  uint16_t    m_asn;

private:
  Ptr<ConnectionManager> m_connectionManager;
};

}

#endif /* CLIENT_DATABASE_H */

