/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef CONNECTION_MANAGER_H
#define CONNECTION_MANAGER_H

#include "ns3/object.h"
#include "ns3/socket.h"

#include "ns3/BitTorrentClient.h"
#include "ns3/ClientDatabase.h"
#include "ns3/PeerConnection.h"

namespace ns3 {

class ListeningPeerConnection;
class PeerConnection;

class ConnectionStrategyBase;
class UnchokeStrategyBase;
class ChokeStrategyBase;

class ConnectionManager : public Object
{
public:
  ConnectionManager(Ptr<BitTorrentClient> client, Ptr<ClientDatabase> clientDB);
private:
  ConnectionManager();

public:
  bool StartConnectionManager(void);
  bool StopConnectionManager(void);

public:
  Ptr<Socket> getListeningSocket(void);
  Ipv4Address getListeningIpv4Address(void);

  Ptr<Socket> CreateTcpSocket(void);
  bool isConnected(std::string peerId);
  uint32_t RestConnectableSocket(void);
  bool isAcceptableNewConnection(void);

  bool EntrustedPeerConnection(Ptr<Socket> socket, std::string peerId);
  void RemoveFromPeerList(Ptr<PeerConnection> peerConnection);

  // PeerConnection
  void ReceiveNewPiece(uint32_t pieceIndex, uint32_t begin, const std::string& message);
  // ClientDatabase
  void NotifiedGetNewPiece(uint32_t pieceIndex);

public:
  bool Choke(Ptr<PeerConnection> peer);

  uint32_t UnchokeMax(void) const;
  bool isUnchoking(Ptr<PeerConnection> peer);
  bool isUnchokable(Ptr<PeerConnection> peer);
  bool Unchoke(Ptr<PeerConnection> peer);

public:
  const Ptr<ClientDatabase>& getClientDatabase(void) const;

  const std::list<Ptr<PeerConnection> >& getConnectedPeerList(void) const;
  const std::list<Ptr<PeerConnection> >& getUnchokingPeerList(void) const;

private:
  void BroadcastHaveMessage(uint32_t pieceIndex);
  void CloseAllConnection(void);
  void CONNECTION_MANAGER_LOG_INFO(std::string info);

private:
  Ptr<Socket> m_listeningSocket;

  Ptr<BitTorrentClient> m_client;
  Ptr<ClientDatabase>   m_clientDB;

  Ptr<ListeningPeerConnection> m_listeningPeerConnection;

  Ptr<ConnectionStrategyBase> m_connectionStrategy;
  Ptr<UnchokeStrategyBase>    m_unchokeStrategy;
  Ptr<ChokeStrategyBase>      m_chokeStrategy;

  std::list<Ptr<PeerConnection> > m_connectedPeerList;
  std::list<Ptr<PeerConnection> > m_unchokingPeerList;

  uint32_t m_nConnectionMax;
  uint32_t m_nUnchokeMax;

};

}

#endif /* CONNECTION_MANAGER_H */

