/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef PEER_CONNECTION_H
#define PEER_CONNECTION_H

#include "ns3/object.h"
#include "ns3/ConnectionManager.h"
#include "ns3/PeerInformation.h"

namespace ns3 {

class Socket;
class Packet;
class RequestStrategyBase;

class PeerConnection : public Object
{
public:
  PeerConnection(Ptr<Socket> socket, std::string peerId, Ptr<ConnectionManager> connectionManager);
private:
  PeerConnection();

public:
  void HandleRead(Ptr<Socket> socket);
  void HandlePeerClose(Ptr<Socket> socket);
  void HandlePeerError(Ptr<Socket> socket);

public:
  void SendChokeMessage(void);
  void SendUnchokeMessage(void);
  void SendHaveMessage(uint32_t pieceIndex);
  void SendRequestMessage(uint32_t pieceIndex, uint32_t begin, uint32_t requestLength);

  void CloseConnection(void);

  std::string getPeerId(void);
  const Ptr<PeerInformation>& getPeerInformation(void);
  const Ptr<ConnectionManager>& getConnectionManager(void);

private:
  void RecvChokeMessage(std::string message);
  void RecvUnchokeMessage(std::string message);
  void RecvInterestedMessage(std::string message);
  void RecvNotInterestedMessage(std::string message);
  void RecvHaveMessage(std::string message);
  void RecvBitfieldMessage(std::string message);
  void RecvRequestMessage(Ptr<Socket> socket, std::string message);
  void RecvPieceMessage(std::string message);
  void RecvCancelMessage(std::string message);
  void RecvKeepAliveMessage(Ptr<Socket> socket, std::string message);

private:
  bool GenerateRandomString(std::string* str, uint32_t length);
  void PEER_CONNECTION_LOG_INFO(std::string info);

private:
  Ptr<ConnectionManager> m_connectionManager;
  Ptr<Socket>            m_socket;
  Ptr<Packet>            m_packet;

  Ptr<PeerInformation>   m_peerInfo;

  Ptr<RequestStrategyBase> m_requestStrategy;

  bool m_waitKeepAliveResponse;
};

}

#endif /* PEER_CONNECTION_H */

