/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef CONNECTION_STRATEGY_BASE_H
#define CONNECTION_STRATEGY_BASE_H

#include <map>
#include "ns3/object.h"
#include "ns3/socket.h"
#include "ns3/packet.h"

#include "ns3/ConnectionManager.h"
#include "ns3/PeerInformation.h"

namespace ns3 {

class ConnectionStrategyBase : public Object
{
public:
  virtual bool StartConnect() = 0;
  virtual bool StopConnect() = 0;

protected:
  bool isConnectable(const PeerInformation& pi);
  bool Connect(PeerInformation& pi);

private:
  void EntrustPeerConnection(Ptr<Socket> socket, std::string peerId);
  void CONNECTION_LOG_INFO(std::string info);

public:
  void ConnectionSucceeded(Ptr<Socket> socket);
  void ConnectionFailed(Ptr<Socket> socket);
  void HandleRead(Ptr<Socket> socket);

protected:
  Ptr<ConnectionManager> m_connectionManager;

private:
  std::map<Ptr<Socket>, Ptr<Packet> > m_connectedSocketList;
};

}

#endif /* CONNECTION_STRATEGY_BASE_H */

