/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include <random>
#include <algorithm>

#include "ns3/log.h"

#include "FullRandomConnectionStrategy.h"

namespace ns3 {

NS_LOG_COMPONENT_DEFINE("FullRandomConnectionStrategy");
NS_OBJECT_ENSURE_REGISTERED(FullRandomConnectionStrategy);

FullRandomConnectionStrategy::FullRandomConnectionStrategy(Ptr<ConnectionManager> connectionManager)
{
  NS_LOG_FUNCTION(this);

  m_connectionManager = connectionManager;
  m_interval = Seconds(1.);
}

FullRandomConnectionStrategy::FullRandomConnectionStrategy()
{
  NS_LOG_FUNCTION(this);

}

bool
FullRandomConnectionStrategy::StartConnect()
{
  NS_LOG_FUNCTION(this);

  m_nextConnectStrategy = Simulator::ScheduleNow(&FullRandomConnectionStrategy::ConnectStrategy, this);
  return true;
}

bool
FullRandomConnectionStrategy::StopConnect()
{
  NS_LOG_FUNCTION(this);

  if (m_nextConnectStrategy.IsRunning())
    {
      Simulator::Cancel(m_nextConnectStrategy);
    }
  return true;
}

void
FullRandomConnectionStrategy::ConnectStrategy()
{
  NS_LOG_FUNCTION(this);

  if (FullRandomConnect())
    {
    }
  else
    {
    }

  m_nextConnectStrategy = Simulator::Schedule(m_interval, &FullRandomConnectionStrategy::ConnectStrategy, this);
}

bool
FullRandomConnectionStrategy::FullRandomConnect()
{
  NS_LOG_FUNCTION(this);

  std::vector<PeerInformation> peerList(m_connectionManager->getClientDatabase()->getPeerList());

  if (peerList.size() == 0)
    return false;

  std::random_device rd;
  std::mt19937 mt(rd());
  std::shuffle(peerList.begin(), peerList.end(), mt);

  for (std::vector<PeerInformation>::iterator it = peerList.begin();
       it != peerList.end(); ++it)
    {
      if (isConnectable(*it))
        {
          Connect(*it);
          return true;
        }
    }
  return false;
}



}

