/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include <random>
#include <algorithm>

#include "ns3/log.h"

#include "FullRandomRequestStrategy.h"

namespace ns3 {

NS_LOG_COMPONENT_DEFINE("FullRandomRequestStrategy");
NS_OBJECT_ENSURE_REGISTERED(FullRandomRequestStrategy);

FullRandomRequestStrategy::FullRandomRequestStrategy(Ptr<PeerConnection> peerConnection)
{
  NS_LOG_FUNCTION(this);

  m_peerConnection = peerConnection;
  m_interval = Seconds(1.);
}

FullRandomRequestStrategy::FullRandomRequestStrategy()
{
  NS_LOG_FUNCTION(this);

}

FullRandomRequestStrategy::~FullRandomRequestStrategy()
{
  NS_LOG_FUNCTION(this);

  StopRequest();
}

bool
FullRandomRequestStrategy::StartRequest(void)
{
  NS_LOG_FUNCTION(this);

  m_nextRequestStrategy = Simulator::ScheduleNow(&FullRandomRequestStrategy::RequestStrategy, this);
  return true;
}

bool
FullRandomRequestStrategy::StopRequest(void)
{
  NS_LOG_FUNCTION(this);

  if (m_nextRequestStrategy.IsRunning())
    {
      Simulator::Cancel(m_nextRequestStrategy);
    }
  return true;
}

void
FullRandomRequestStrategy::RequestStrategy(void)
{
  if (FullRandomRequest())
    {
    }
  else
    {
    }
  m_nextRequestStrategy = Simulator::Schedule(m_interval, &FullRandomRequestStrategy::RequestStrategy, this);
}

bool
FullRandomRequestStrategy::FullRandomRequest(void)
{
  const std::vector<bool>& bitfield   = m_peerConnection->getPeerInformation()->getBitfield();
  const std::vector<bool>& myBitfield = m_peerConnection->getConnectionManager()->getClientDatabase()->getBitfield();

  if (bitfield.size() != myBitfield.size())
    {
      NS_LOG_INFO("FullRandomRequestStrategy: bitfield size is wrong");

      return false;
    }

  // generate candidate piece list
  std::vector<uint32_t> candidateList;
  for (uint32_t i=0; i<bitfield.size(); i++)
    {
      if ((bitfield.at(i) == true) && (myBitfield.at(i) == false))
        candidateList.push_back(i);
    }

  if (candidateList.size() == 0)
    return false;

  // fullrandom select piece
  std::random_device rd;
  std::mt19937 mt(rd());
  std::shuffle(candidateList.begin(), candidateList.end(), mt);

  uint32_t pieceLength = m_peerConnection->getConnectionManager()->getClientDatabase()->getPieceLength();

  // send first pieceIndex
  return Request(candidateList.at(0), 0, pieceLength);
}


}

