/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include <random>
#include <algorithm>

#include "ns3/log.h"

#include "FullRandomUnchokeStrategy.h"

namespace ns3 {

NS_LOG_COMPONENT_DEFINE("FullRandomUnchokeStrategy");
NS_OBJECT_ENSURE_REGISTERED(FullRandomUnchokeStrategy);

FullRandomUnchokeStrategy::FullRandomUnchokeStrategy(Ptr<ConnectionManager> connectionManager)
{
  NS_LOG_FUNCTION(this);

  m_connectionManager = connectionManager;
  m_interval = Seconds(1.);
}

FullRandomUnchokeStrategy::FullRandomUnchokeStrategy()
{
  NS_LOG_FUNCTION(this);

}

bool
FullRandomUnchokeStrategy::StartUnchoke()
{
  NS_LOG_FUNCTION(this);

  m_nextUnchokeStrategy = Simulator::ScheduleNow(&FullRandomUnchokeStrategy::UnchokeStrategy, this);
  return true;
}

bool
FullRandomUnchokeStrategy::StopUnchoke()
{
  NS_LOG_FUNCTION(this);

  if (m_nextUnchokeStrategy.IsRunning())
    {
      Simulator::Cancel(m_nextUnchokeStrategy);
    }
  return true;
}

void
FullRandomUnchokeStrategy::UnchokeStrategy()
{
  NS_LOG_FUNCTION(this);

  if (FullRandomUnchoke())
    {
    }
  else
    {
    }
  m_nextUnchokeStrategy = Simulator::Schedule(m_interval, &FullRandomUnchokeStrategy::UnchokeStrategy, this);
}

bool
FullRandomUnchokeStrategy::FullRandomUnchoke()
{
  NS_LOG_FUNCTION(this);

  //MEMO: std::shuffle can't treat std::list
  const std::list<Ptr<PeerConnection> > connectedPeerList(m_connectionManager->getConnectedPeerList());
  std::vector<Ptr<PeerConnection> > candidateList(connectedPeerList.begin(), connectedPeerList.end());

  if (candidateList.size() == 0)
    return false;
  
  std::random_device rd;
  std::mt19937 mt(rd());
  std::shuffle(candidateList.begin(), candidateList.end(), mt);

  for (std::vector<Ptr<PeerConnection> >::iterator it = candidateList.begin();
       it != candidateList.end(); ++it)
    {
      if (isUnchokable(*it))
        {
          return Unchoke(*it);
        }
    }
  return false;

}

}

