/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include <random>
#include <algorithm>

#include "SimpleChokeStrategy.h"

namespace ns3 {

SimpleChokeStrategy::SimpleChokeStrategy(Ptr<ConnectionManager> connectionManager)
{
  m_connectionManager = connectionManager;
  m_interval = Seconds(30.);
}

SimpleChokeStrategy::SimpleChokeStrategy()
{
}

bool
SimpleChokeStrategy::StartChoke(void)
{
  m_nextChokeStrategy = Simulator::Schedule(m_interval, &SimpleChokeStrategy::ChokeStrategy, this);
  return true;
}

bool
SimpleChokeStrategy::StopChoke(void)
{
  if (m_nextChokeStrategy.IsRunning())
    {
      Simulator::Cancel(m_nextChokeStrategy);
    }
  return true;
}

void
SimpleChokeStrategy::ChokeStrategy(void)
{
  if (FullRandomChoke())
    {
    }
  else
    {
    }
  m_nextChokeStrategy = Simulator::Schedule(m_interval, &SimpleChokeStrategy::ChokeStrategy, this);
}

bool
SimpleChokeStrategy::FullRandomChoke(void)
{
  const std::list<Ptr<PeerConnection> > chokingPeerList(m_connectionManager->getUnchokingPeerList());
  if (chokingPeerList.size() != m_connectionManager->UnchokeMax())
    return false;

  std::vector<Ptr<PeerConnection> > candidateList(chokingPeerList.begin(), chokingPeerList.end());

  std::random_device rd;
  std::mt19937 mt(rd());
  std::shuffle(candidateList.begin(), candidateList.end(), mt);

  return Choke(candidateList.at(0));
}

}

