/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include <map>
#include <random>
#include <algorithm>

#include "WeightedRandomUnchokeStrategy.h"

namespace ns3 {

WeightedRandomUnchokeStrategy::WeightedRandomUnchokeStrategy(Ptr<ConnectionManager> connectionManager)
{
  m_connectionManager = connectionManager;
  m_interval = Seconds(1.);

  m_weight = 10;
}

WeightedRandomUnchokeStrategy::WeightedRandomUnchokeStrategy()
{
}

bool
WeightedRandomUnchokeStrategy::StartUnchoke()
{
  m_nextUnchokeStrategy = Simulator::ScheduleNow(&WeightedRandomUnchokeStrategy::UnchokeStrategy, this);
  return true;
}

bool
WeightedRandomUnchokeStrategy::StopUnchoke()
{
  if (m_nextUnchokeStrategy.IsRunning())
    {
      Simulator::Cancel(m_nextUnchokeStrategy);
    }
  return true;
}

void
WeightedRandomUnchokeStrategy::UnchokeStrategy()
{
  if (WeightedRandomUnchoke())
    {
    }
  else
    {
    }
  m_nextUnchokeStrategy = Simulator::Schedule(m_interval, &WeightedRandomUnchokeStrategy::UnchokeStrategy, this);
}

bool
WeightedRandomUnchokeStrategy::WeightedRandomUnchoke()
{
  // generate ASN table
  std::map<std::string, uint16_t> asnTable;
  const std::vector<PeerInformation>& peerList = m_connectionManager->getClientDatabase()->getPeerList();
  for (std::vector<PeerInformation>::const_iterator it = peerList.begin();
       it != peerList.end(); ++it)
    {
      asnTable[it->getPeerId()] = it->getAsn();
    }

  const uint16_t myAsn = m_connectionManager->getClientDatabase()->getAsn();

  // generate weighted candidate list
  std::vector<Ptr<PeerConnection> > candidateList;

  for (std::list<Ptr<PeerConnection> >::const_iterator it = m_connectionManager->getConnectedPeerList().begin();
       it != m_connectionManager->getConnectedPeerList().end(); ++it)
    {
      std::string peerId = (*it)->getPeerInformation()->getPeerId();

      if (asnTable.at(peerId) == myAsn)
        {
          for (uint32_t i=0; i<m_weight; i++)
            candidateList.push_back(*it);
        }
      else
        {
          candidateList.push_back(*it);
        }
    }

  if (candidateList.size() == 0)
    return false;
  
  std::random_device rd;
  std::mt19937 mt(rd());
  std::shuffle(candidateList.begin(), candidateList.end(), mt);

  for (std::vector<Ptr<PeerConnection> >::iterator it = candidateList.begin();
       it != candidateList.end(); ++it)
    {
      if (isUnchokable(*it))
        {
          return Unchoke(*it);
        }
    }
  return false;

}

}

