/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef BENCODING_OBJECT_H
#define BENCODING_OBJECT_H

#include <map>
#include "ns3/object.h"

namespace ns3{

class BencodingObjectIterator;

class BencodingObject : public Object
{
public:
  typedef enum {INTEGER, STRING, LIST, DICTIONARY, NULL_OBJECT} ObjectType;
  typedef std::list<Ptr<BencodingObject> >::iterator listIterator;
  typedef std::map<std::string, Ptr<BencodingObject> >::iterator dictionaryIterator;

public:
  static Ptr<BencodingObject> ParseBencodingObject(uint8_t* data, uint32_t length) throw (std::string);

  BencodingObject();
  BencodingObject(BencodingObject::ObjectType type);
  BencodingObject(long integer);
  BencodingObject(std::string string);

  ~BencodingObject();
  void clearObject();

  // ObjectType
  void changeObjectType(BencodingObject::ObjectType newType);
  BencodingObject::ObjectType getObjectType(void);

  // Accesser member functions
  void setInteger(long integer) throw (std::string);
  long getInteger(void) throw (std::string);

  void setString(std::string string) throw (std::string);
  std::string getString(void) throw (std::string);

  void addToList(Ptr<BencodingObject> item) throw (std::string);
  int getListSize(void) throw (std::string);
  BencodingObject::listIterator listBegin(void) throw (std::string);
  BencodingObject::listIterator listEnd(void) throw (std::string);
  BencodingObject::listIterator eraseFromList(BencodingObject::listIterator it) throw (std::string);

  void addToDictionary(std::string key, Ptr<BencodingObject> value) throw (std::string);
  int getDictionarySize(void) throw (std::string);
  BencodingObject::dictionaryIterator dictionaryBegin(void) throw (std::string);
  BencodingObject::dictionaryIterator dictionaryEnd(void) throw (std::string);
  BencodingObject::dictionaryIterator eraseFromDictionary(BencodingObject::dictionaryIterator it) throw (std::string);
  Ptr<BencodingObject> getValueFromDictionary(std::string key) throw (std::string);

  uint32_t serialize(uint8_t* buf, const uint32_t length) throw (std::string);

private:
  static Ptr<BencodingObject> ParseBencodingObject(uint8_t* data, uint32_t length, uint32_t* cursor) throw (std::string);

  ObjectType m_type = BencodingObject::NULL_OBJECT;

  long         m_integer = 0;
  std::string  m_string  = "";

  std::list<Ptr<BencodingObject> >*             m_list       = NULL;
  std::map<std::string, Ptr<BencodingObject> >* m_dictionary = NULL;
};

}


#endif /* BENCODING_OBJECT_H */
