/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#ifndef BITTORRENT_TRACKER_SERVER_H
#define BITTORRENT_TRACKER_SERVER_H

#include "ns3/application.h"
#include "ns3/event-id.h"
#include "ns3/ptr.h"
#include "ns3/traced-callback.h"
#include "ns3/BencodingObject.h"

namespace ns3 {

class Socket;

class BitTorrentTracker : public Application
{
public:
  BitTorrentTracker();
  virtual ~BitTorrentTracker();

  void SetListeningPort(uint16_t port);

protected:
  virtual void DoDispose(void);

private:
  virtual void StartApplication(void);
  virtual void StopApplication(void);

  void HandleRead(Ptr<Socket> socket);
  void HandleAccept(Ptr<Socket> socket, const Address& from);
  void HandlePeerClose(Ptr<Socket> socket);
  void HandlePeerError(Ptr<Socket> socket);

  void AddToPeerList(std::string peerId, std::string ipAddress, std::string port, std::string asn);
  void RemoveFromPeerList(std::string peerId);

  void SendPeerList(Ptr<Socket> socket);
  void SendFailureMessage(Ptr<Socket> socket);

  uint16_t    m_listeningPort;
  Ptr<Socket> m_socket; // Listening socket
  std::list<Ptr<Socket> > m_socketList; // the accepted sockets

  Ptr<BencodingObject> m_peerList;
};

} // namespace ns3

#endif /* BITTORRENT_TRACKER_SERVER_H */

